#include <stdio.h>
#include <unistd.h>

// Harpertown values
#define PEAK_flops_per_cycle 4
#define cycles_per_sec 3e9

// #repertitions
#define MAX_ITER 100000000

// array length
#define LEN 5

#define rdtscll(val) do { \
	unsigned int __a, __d; \
    asm volatile("rdtsc" : "=a" (__a), "=d" (__d)); \
    (val) = ((unsigned long) __a) | (((unsigned long) __d) << 32); \
} while(0)

int main(){
	int i, j;
	double a[LEN], b[LEN], c[LEN];

	// initialize
	for (j = 0; j < LEN; j++) {
        a[j] = 1 + (j + 1) * 1.1e-6;
        b[j] = 1 + (j + 1) * 1.2e-6;
        c[j] = 1 + (j + 1) * 1.3e-6;
    }

    // compute
	long long int cycles_start, cycles_end;
	rdtscll(cycles_start);
	for (i = 0; i < MAX_ITER; i++)
		for (j = 0; j < LEN; j++) {
			a[j] += b[j];
            b[j] *= c[j];
        }
	rdtscll(cycles_end);

	// dummy output
	for (j = 1; j < LEN; j++)
		a[0] += a[j];
	printf("%1.1g\r", a[0]);

    // calculate performance
	long long int cycles = cycles_end - cycles_start;
	long long int flops = (long long int) MAX_ITER * LEN * 2;
	double flops_per_cycle = (double) flops / cycles;
	double efficiency = flops_per_cycle / PEAK_flops_per_cycle;
	double time = cycles / cycles_per_sec;
	double flops_per_sec = flops / time;

	printf("#cycles:    %lld\n", cycles);
	printf("#ops:       %lld\n", flops);
	printf("#ops/cycle: %g\n", flops_per_cycle);
	printf("efficiency: %g %%\n", 100 * efficiency);
	printf("time:       %g s\n", time);
	printf("GFLOPS:     %g\n", flops_per_sec / 1e9);

	return 0;
}
