#include <stdio.h>
#include <unistd.h>

// Harpertown values
#define PEAK_flops_per_cycle 4
#define cycles_per_sec 3e9

// #repertitions
#define MAX_ITER 100000000

// array length
#define LEN 5
// vector length
#define VLEN 2

#define rdtscll(val) do { \
	unsigned int __a, __d; \
    asm volatile("rdtsc" : "=a" (__a), "=d" (__d)); \
    (val) = ((unsigned long) __a) | (((unsigned long) __d) << 32); \
} while(0)

typedef union {
    double __attribute__((vector_size(8 * VLEN))) v;
    double d[VLEN];
} vdouble;

int main(){
	int i, j, k;
	vdouble a[LEN], b[LEN], c[LEN];

	// initialize
	for (j = 0; j < LEN; j++)
        for (k = 0; k < VLEN; k++) { 
            a[j].d[k] = 1 + (VLEN * j + k + 1) * 1.1e-6;
            b[j].d[k] = 1 + (VLEN * j + k + 1) * 1.2e-6;
            c[j].d[k] = 1 + (VLEN * j + k + 1) * 1.3e-6;
        }

    // compute
	long long int cycles_start, cycles_end;
	rdtscll(cycles_start);
	for (i = 0; i < MAX_ITER; i++)
		for (j = 0; j < LEN; j++) {
			a[j].v += b[j].v;
			b[j].v *= c[j].v;
        }
	rdtscll(cycles_end);

	// dummy output
	for (j = 1; j < LEN; j++)
		a[0].v += a[j].v;
	for (k = 1; k < VLEN; k++)
		a[0].d[0] += a[0].d[k];
	printf("%1.1g\r", a[0].d[0]);

    // calculate performance
	long long int cycles = cycles_end - cycles_start;
	long long int flops = (long long int) MAX_ITER * LEN * VLEN * 2;
	double flops_per_cycle = (double) flops / cycles;
	double efficiency = flops_per_cycle / PEAK_flops_per_cycle;
	double time = cycles / cycles_per_sec;
	double flops_per_sec = flops / time;

	printf("#cycles:    %lld\n", cycles);
	printf("#ops:       %lld\n", flops);
	printf("#ops/cycle: %g\n", flops_per_cycle);
	printf("efficiency: %g %%\n", 100 * efficiency);
	printf("time:       %g s\n", time);
	printf("GFLOPS:     %g\n", flops_per_sec / 1e9);

	return 0;
}
